import * as d3 from 'd3';
class HeatMap {
    constructor(cube, elementId) {
        this.cube = cube;
        this.elementId = elementId;
    }

    init() {
        const margin = { top: 50, right: 0, bottom: 100, left: 30 },
            width = 960 - margin.left - margin.right,
            height = 430 - margin.top - margin.bottom,
            gridSize = Math.floor(width / 24),
            legendElementWidth = gridSize*2,
            buckets = 9,
            colors = ["#ffffd9","#edf8b1","#c7e9b4","#7fcdbb","#41b6c4","#1d91c0","#225ea8","#253494","#081d58"], // alternatively colorbrewer.YlGnBu[9]
            days = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"],
            times = ["12am", "1am", "2am", "3am", "4am", "5am", "6am", "7am", "8am", "9am", "10am", "11am", "12pm", "1pm", "2pm", "3pm", "4pm", "5pm", "6pm", "7pm", "8pm", "9pm", "10pm", "11pm"];

        const svg = d3.select(this.elementId).append("svg")
            .attr("width", width + margin.left + margin.right)
            .attr("height", height + margin.top + margin.bottom)
            .append("g")
            .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

        const dayLabels = svg.selectAll(".dayLabel")
            .data(days)
            .enter().append("text")
            .text(function (d) { return d; })
            .attr("x", 0)
            .attr("y", (d, i) => i * gridSize)
            .style("text-anchor", "end")
            .attr("transform", "translate(-6," + gridSize / 1.5 + ")")
            .attr("class", (d, i) => ((i >= 0 && i <= 4) ? "dayLabel mono axis axis-workweek" : "dayLabel mono axis"));

        const timeLabels = svg.selectAll(".timeLabel")
            .data(times)
            .enter().append("text")
            .text((d) => d)
            .attr("x", (d, i) => i * gridSize)
            .attr("y", 0)
            .style("text-anchor", "middle")
            .attr("transform", "translate(" + gridSize / 2 + ", -6)")
            .attr("class", (d, i) => ((i >= 7 && i <= 16) ? "timeLabel mono axis axis-worktime" : "timeLabel mono axis"));

        const type = (d) => {
            return {
                day: +d.day,
                hour: +d.hour,
                value: +d.value
            };
        };

        this.svg = svg;
        this.width = width;
        this.height = height;
        this.gridSize = gridSize;
        this.legendElementWidth = legendElementWidth;
        this.buckets = buckets;
        this.colors = colors;
        //listen for changes on object, and update
        this.cube.object.on('changed', () => {
            this.cube.object.getLayout().then((layout) => {
                this.layout = layout;
                this.update();
            });
        });
        //call update manually on init to populate dropdown
        this.cube.object.getLayout().then((layout) => {
            this.layout = layout;
            this.update();
        });
    }

    update() {
        let svg = this.svg;
        let width = this.width;
        let height = this.height;
        let gridSize = this.gridSize;
        let legendElementWidth = this.legendElementWidth;
        let buckets = this.buckets;
        let colors = this.colors;
        let layout = this.layout;

        const colorScale = d3.scaleQuantile()
        .domain([0, buckets - 1, layout.qHyperCube.qMeasureInfo[0].qMax])
        .range(colors);

        svg.selectAll(".hour").remove();
        const cards = svg.selectAll(".hour")
            .data(layout.qHyperCube.qDataPages[0].qMatrix); //.data(data, (d) => d.day+':'+d.hour);

        cards.append("title");

        cards.enter().append("rect")
            .attr("x", (d) => (d[1].qNum) * gridSize)
            .attr("y", (d) => (d[0].qNum) * gridSize)
            .attr("rx", 4)
            .attr("ry", 4)
            .attr("class", "hour bordered")
            .attr("width", gridSize)
            .attr("height", gridSize)
            .style("fill", colors[0])
        .merge(cards)
            .transition()
            .duration(1000)
            .style("fill", (d) => colorScale(d[2].qNum));

        cards.select("title").text((d) => d[2].qNum);

        cards.exit().remove();

        const legend = svg.selectAll(".legend")
            .data([0].concat(colorScale.quantiles()), (d) => d);

        const legend_g = legend.enter().append("g")
            .attr("class", "legend");

        legend_g.append("rect")
        .attr("x", (d, i) => legendElementWidth * i)
        .attr("y", height)
        .attr("width", legendElementWidth)
        .attr("height", gridSize / 2)
        .style("fill", (d, i) => colors[i]);

        legend_g.append("text")
        .attr("class", "mono")
        .text((d) => "≥ " + Math.round(d))
        .attr("x", (d, i) => legendElementWidth * i)
        .attr("y", height + gridSize);

        legend.exit().remove();
    }

}
export default HeatMap;